<?php

namespace Drupal\etracker\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\etracker\Helper\Constants;
use Drupal\etracker\Helper\Functions;

/**
 * Configure eTracker settings for this site.
 */
class EtrackerAdminSettingsForm extends ConfigFormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'etracker_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      Constants::ETRACKER_SETTINGS_CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(Constants::ETRACKER_SETTINGS_CONFIG_NAME);

    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General settings'),
      '#open' => TRUE,
    ];

    $form['general']['account_key'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Account key'),
      '#description' => $this->t('To find your account key 1, go to your <a href="https://newapp.etracker.com/#/report/accountSettings/accountKey">etracker account</a>.'),
      '#default_value' => $config->get('account_key'),
    ];

    // Scope
    $form['general']['etracker_scope_script'] = [
      '#type' => 'radios',
      '#title' => $this->t('Include script into'),
      '#options' => [
        Constants::ETRACKER_SCOPE_SCRIPT_HEADER => $this->t('Header') . ' (' . $this->t('recommended') . ')',
        Constants::ETRACKER_SCOPE_SCRIPT_FOOTER => $this->t('Footer'),
      ],
      '#default_value' => $config->get('etracker_scope_script') != NULL ? $config->get('etracker_scope_script') : Constants::ETRACKER_SCOPE_SCRIPT_HEADER,
      '#description' => $this->t('The JavaScript is built in a way that it does <strong>not</strong> block or delay the loading of the page and to support all functionality of the etracker suite it is required to have the etracker tracking code in the header of the page. Only change this if really needed.'),
    ];

    // Tracking settings.
    // Vertical tabs wrapper.
    $form['tracking_scope'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Tracking'),
    ];

    // Specific pages
    $form['tracking']['specific_pages'] = [
      '#type' => 'details',
      '#title' => $this->t('Specific pages'),
      '#group' => 'tracking_scope',
    ];

    $track_path_mode = $config->get('etracker_track_path_mode');

    $options = [
      Constants::ETRACKER_TRACK_PATHS_MODE_ALL => $this->t('All pages except those listed'),
      Constants::ETRACKER_TRACK_PATHS_MODE_LISTED => $this->t('Only the listed pages'),
    ];
    $description = $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.",
      [
        '%blog' => 'blog',
        '%blog-wildcard' => 'blog/*',
        '%front' => '<front>',
      ]);


    $title = $this->t('Pages');

    $form['tracking']['specific_pages']['etracker_track_path_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Tracking specific pages'),
      '#options' => $options,
      '#default_value' => $track_path_mode != NULL ? $track_path_mode : Constants::ETRACKER_TRACK_PATHS_MODE_ALL,
    ];
    $form['tracking']['specific_pages']['etracker_track_paths'] = [
      '#type' => 'textarea',
      '#title' => '<span class="element-invisible">' . $title . '</span>',
      '#default_value' => $config->get('etracker_track_paths') != NULL ? $config->get('etracker_track_paths') : Constants::ETRACKER_TRACK_PATHS,
      '#description' => $description,
    ];


    // Roles
    $form['tracking']['roles'] = [
      '#type' => 'details',
      '#title' => $this->t('Roles'),
      '#group' => 'tracking_scope',
    ];

    $track_roles_mode = $config->get('etracker_track_roles_mode');
    $form['tracking']['roles']['etracker_track_roles_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Tracking specific roles'),
      '#options' => [
        Constants::ETRACKER_TRACK_ROLES_MODE_ALL => $this->t('All roles except those listed'),
        Constants::ETRACKER_TRACK_ROLES_MODE_LISTED => $this->t('Only the selected roles'),
      ],
      '#default_value' => $track_roles_mode != NULL ? $track_roles_mode : Constants::ETRACKER_TRACK_ROLES_MODE_ALL,
    ];

    $role_options = user_role_names();
    if (sizeof($role_options) > 0) {
      $form['tracking']['roles']['etracker_track_roles'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Roles'),
        '#options' => $role_options,
        '#default_value' => $config->get('etracker_track_roles') ?? [],
      ];
    }

    // User
    $form['tracking']['user'] = [
      '#type' => 'details',
      '#title' => $this->t('User'),
      '#group' => 'tracking_scope',
    ];

    $t_permission = ['%permission' => $this->t('opt-in or out of etracker tracking')];
    $form['tracking']['user']['etracker_track_user'] = [
      '#type' => 'radios',
      '#title' => $this->t('Allow users to customize tracking on their account page'),
      '#options' => [
        Constants::ETRACKER_TRACK_USER_NO_CUSTOMIZATION => $this->t('No customization allowed'),
        Constants::ETRACKER_TRACK_USER_TRACKING_ON => $this->t('Tracking on by default, users with %permission permission can opt out',
          $t_permission),
        Constants::ETRACKER_TRACK_USER_TRACKING_OFF => $this->t('Tracking off by default, users with %permission permission can opt in',
          $t_permission),
      ],
      '#default_value' => $config->get('etracker_track_user') != NULL ? $config->get('etracker_track_user') : Constants::ETRACKER_TRACK_USER_NO_CUSTOMIZATION,
    ];

    //Default values
    $form['default_values'] = [
      '#type' => 'details',
      '#title' => $this->t('Default values'),
      '#open' => TRUE,
    ];

    $form['default_values']['etracker_mode_breadcrumb_as_area'] = [
      '#type' => 'radios',
      '#title' => $this->t('Breadcrumb as area'),
      '#default_value' => $config->get('etracker_mode_breadcrumb_as_area') != NULL ? $config->get('etracker_mode_breadcrumb_as_area') : Constants::ETRACKER_MODE_BREADCRUMB_ON_EXCLUDE_HOME,
      '#description' => $this->t('Select if and how you want to utilize your breadcrumb as an hierarchical area for tracking. If turned on you can select whether the root of your site will be used as the most top area of the hierarchy or not.'),
      '#options' => [
        Constants::ETRACKER_MODE_BREADCRUMB_OFF => $this->t('Off'),
        Constants::ETRACKER_MODE_BREADCRUMB_ON => $this->t('On'),
        Constants::ETRACKER_MODE_BREADCRUMB_ON_EXCLUDE_HOME => $this->t('On, exclude home'),
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable(Constants::ETRACKER_SETTINGS_CONFIG_NAME);

    if ($config->get('account_key') != $form_state->getValue('account_key')){
      Cache::invalidateTags(["library_info"]);
    }

    $config->set('account_key', $form_state->getValue('account_key'));
    $config->set('etracker_scope_script',
      $form_state->getValue('etracker_scope_script'));

    //Specific Pages
    $config->set('etracker_track_path_mode',
      $form_state->getValue('etracker_track_path_mode'));
    $config->set('etracker_track_paths',
      $form_state->getValue('etracker_track_paths'));

    //Roles
    $config->set('etracker_track_roles_mode',
      $form_state->getValue('etracker_track_roles_mode'));
    $config->set('etracker_track_roles',
      $form_state->getValue('etracker_track_roles'));

    //User
    $config->set('etracker_track_user',
      $form_state->getValue('etracker_track_user'));

    //Default Values
    $config->set('etracker_mode_breadcrumb_as_area',
      $form_state->getValue('etracker_mode_breadcrumb_as_area'));

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
